/* eslint-env node */
'use strict';

// Ce module nodejs gère l'API de notre site
// Il définit l'ensemble des routes (relative à "/api") corresponant aux 
// points d'entrée de l'API

// Expressjs
const express = require('express');
// Notre module nodejs d'accès simplifié à la base de données
const dbHelper = require('./dbhelper.js');

// Comme c'est un module nodejs il faut exporter les fonction qu'on veut rendre publiques
// ici on n'exporte qu'ne seule fonction (anonyme) qui est le "constructeur" du module
// Cette fonction prend en paramètre un objet "passport" pour la gestion de l'authentification 
module.exports = (passport) => {
    const app = express();

    // Point d'entrée permettant de récupérer la liste des albums d'un artiste
    // Pas besoin d'authentification pour accéder aux albums on n'utilise pas passport ici
    app.get('/artist/:artist_id/albums', function (req, res, next) {
        dbHelper.artists.byId(req.params.artist_id).albums.then(
            albums => {
                res.set('Content-type', 'application/json');
                res.send(JSON.stringify(albums));
            },
            err => {
                next(err);
            },
        );
    });

    // Point d'entrée permettant de récupérer la liste des artistes
    // Pas besoin d'authentification pour accéder aux albums on n'utilise pas passport ici
    app.get('/artists', function (req, res, next) {
        dbHelper.artists.all().then(
            artists => {
                res.set('Content-type', 'application/json');
                res.send(JSON.stringify(artists));
            },
            err => {
                next(err);
            },
        );
    });

    // Exemple de point d'entré (qui ne fait rien d'intressant) de l'api
    // qui nécessite une authentification.
    // C'est le "require('connect-ensure-login').ensureLoggedIn()" qui vérifie
    // que l'utilisateur est bien authentifié. Si ce n'est pas le cas il sera redirigé
    // vers la page de login
    app.get('/nimportequoi',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            // on fait ce qu'on a a faire (ici on renvoit juste du texte brut "nimp"
            // si l'utilisateur est bien authentifié
            res.send('nimp');
        }
    );



    // Authentification pour accéder aux parties privées de l'api (on n'en a pas dans cet exemple)
    // et aux templates privés
    // C'est ici qu'on utilise passport pour créer une session utilisateur
    app.post('/login', function (req, res, next) {
        if (!req.body.username) {
            return res.send({ success: false, message: 'empty username' });
        }
        if (!req.body.password) {
            return res.send({ success: false, message: 'empty password' });
        }
        passport.authenticate('local', function (err, user) {
            if (err) {
                return next(err); // will generate a 500 error
            }
            if (!user) {
                return res.send({ succes: false, message: 'authentication failed' });
            }
            req.login(user, function (err) {
                if (err) {
                    return next(err);
                }
                return res.send({ success: true, message: 'authentication succeeded' });
            });
        })(req, res, next);
    });



    // TODO gerer les erreurs SQL par des status appropriés (ex 404 si pas trouvé, 40X si patati patata, ou 50X si erreur interne)


    app.get('/',
        // // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            res.set('Content-type', 'text/pain');
            res.status(200).send('API');

        }
    );



    //////////////////////////////////////////////////////////////////////////////
    // CRUD FOURNISSEUR
    //////////////////////////////////////////////////////////////////////////////


    app.get('/fournisseur',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.fournisseur.all()
                .then(
                    fournisseur => {
                        res.set('Content-type', 'application/json');
                        res.status(200).send(JSON.stringify(fournisseur));
                        return;
                    })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );

    app.get('/fournisseur/:nom_fournisseur',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.fournisseur.byNom(req.params.nom_fournisseur)
                .then(fournisseur => {
                    res.set('Content-type', 'application/json');
                    res.status(200).send(JSON.stringify(fournisseur));
                    return;
                })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );

    app.post('/fournisseur',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {

            // Note: Le format utiliser est x-www-form-urlencoded.

            dbHelper.fournisseur.create(
                req.body.nom_fournisseur,
                req.body.adresse,
                req.body.email,
                req.body.telephone,
            ).then(fournisseur => {
                res.set('Content-type', 'plain/text');
                res.status(200).send('fournisseur créé');
                return;
            }).catch(err => {
                res.status(500).send('' + err);
                return;
            });
        }
    );

    app.put('/fournisseur/:nom_fournisseur',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {

            console.log('PUT FOURNISSEUR:');
            console.log('PARAMS: ', req.params);
            console.log('BODY: ', req.body);

            //si body est vide
            if (Object.keys(req.body).length === 0) {
                res.status(400).send('Aucun champs à modifier.');
                return;
            }

            dbHelper.fournisseur.update(
                req.params.nom_fournisseur,
                req.body.nom_fournisseur,
                req.body.adresse,
                req.body.email,
                req.body.telephone,
            ).then(fournisseur => {
                res.set('Content-type', 'plain/text');
                res.status(200).send('fournisseur modifié');
                return;
            }).catch(err => {
                res.status(500).send('' + err);
                return;
            });
        }
    );

    // note : supprimer un fournisseur ne supprimera pas en cascade les produits associés
    // note : ne gère pas les suppressions dans le vide. Elle seront affichées comme réussies mais ne supprimeront rien.
    app.delete('/fournisseur/:nom_fournisseur',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.fournisseur.delete(req.params.nom_fournisseur)
                .then(fournisseur => {
                    res.set('Content-type', 'plain/text');
                    res.status(200).send('fournisseur supprimé');
                    return;
                }).catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );


    //////////////////////////////////////////////////////////////////////////////
    // CRUD PRODUIT
    //////////////////////////////////////////////////////////////////////////////

    app.get('/produit',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.produit.all()
                .then(
                    produit => {
                        res.set('Content-type', 'application/json');
                        res.status(200).send(JSON.stringify(produit));
                        return;
                    })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );

    app.get('/produit/:nom_produit',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.produit.byNom(req.params.nom_produit)
                .then(produit => {
                    res.set('Content-type', 'application/json');
                    res.status(200).send(JSON.stringify(produit));
                    return;
                })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );

    app.post('/produit',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.produit.create(
                req.body.nom_produit,
                req.body.nom_fournisseur,
                req.body.prix_unitaire,
                req.body.quantite_min,
                req.body.quantite_max,
                req.body.image,
                req.body.quantite_course,
            ).then(produit => {
                res.set('Content-type', 'plain/text');
                res.status(200).send('produit créé');
                return;
            }).catch(err => {
                res.status(500).send('' + err);
                return;
            });
        }
    );

    app.put('/produit/:nom_produit',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {

            console.log('PUT PRODUIT:');
            console.log('PARAMS: ', req.params);
            console.log('BODY: ', req.body);

            //si body est vide
            if (Object.keys(req.body).length === 0) {
                res.status(400).send('Aucun champs à modifier.');
                return;
            }

            dbHelper.produit.update(
                req.params.nom_produit,
                req.body.nom_produit,
                req.body.nom_fournisseur,
                req.body.prix_unitaire,
                req.body.quantite_min,
                req.body.quantite_max,
                req.body.image,
                req.body.quantite_course,
            ).then(produit => {
                res.set('Content-type', 'plain/text');
                res.status(200).send('produit modifié');
                return;
            }).catch(err => {
                res.status(500).send('' + err);
                return;
            });
        }
    );

    app.delete('/produit/:nom_produit',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.produit.delete(req.params.nom_produit)
                .then(produit => {
                    res.set('Content-type', 'plain/text');
                    res.status(200).send('produit supprimé');
                    return;
                }).catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );


    //////////////////////////////////////////////////////////////////////////////
    // CRUD STOCK
    //////////////////////////////////////////////////////////////////////////////

    app.get('/stock',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.stock.all()
                .then(
                    stock => {
                        res.set('Content-type', 'application/json');
                        res.status(200).send(JSON.stringify(stock));
                        return;
                    })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );

    app.get('/stock/:nom_produit/:date_peremption',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.stock.byNomDate(req.params.nom_produit)
                .then(stock => {
                    res.set('Content-type', 'application/json');
                    res.status(200).send(JSON.stringify(stock));
                    return;
                })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );

    app.post('/stock',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {

            // TODO si le body ne contient pas au minimum nom_produit et date_peremption. On répond à la requete par une erreur


            dbHelper.stock.create(
                req.body.nom_produit,
                req.body.date_peremption,
                req.body.quantite,
            ).then(stock => {
                res.set('Content-type', 'plain/text');
                res.status(200).send('stock créé');
                return;
            }).catch(err => {
                res.status(500).send('' + err);
                return;
            });
        }
    );

    app.put('/stock/:nom_produit/:date_peremption',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {

            console.log('PUT STOCK:');
            console.log('PARAMS: ', req.params);
            console.log('BODY: ', req.body);

            //si body est vide
            if (Object.keys(req.body).length === 0) {
                res.status(400).send('Aucun champs à modifier.');
                return;
            }

            dbHelper.stock.update(
                req.params.nom_produit,
                req.params.date_peremption,
                req.body.nom_produit,
                req.body.date_peremption,
                req.body.quantite,
            ).then(stock => {
                res.set('Content-type', 'plain/text');
                res.status(200).send('stock modifié');
                return;
            }).catch(err => {
                res.status(500).send('' + err);
                return;
            });
        }
    );

    app.delete('/stock/:nom_produit/:date_peremption',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {
            dbHelper.stock.delete(req.params.nom_produit, req.params.date_peremption)
                .then(stock => {
                    res.set('Content-type', 'plain/text');
                    res.status(200).send('stock supprimé');
                    return;
                }).catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );



    //////////////////////////////////////////////////////////////////////////////
    // Inventaire
    //////////////////////////////////////////////////////////////////////////////

    app.get('/inventaire',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {

            // pour chaque produit, obtenir la liste de ses stocks:
            // {
            // nom_produit: 'jambon',
            // nom_fournisseur: 'leclerc',
            // prix_unitaire: 10,
            // quantite_min: 50,
            // quantite_max: 10,
            // image: 'https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg',
            // quantite_course: 0,
            // stocks: [
            //     {
            //         datePeremption: 2023 - 05 - 12,
            //         quantite_stock: 12,
            //     },
            //     {
            //         datePeremption: 2023 - 05 - 12,
            //         quantite_stock: 12,
            //     },
            // ]
            // },

            dbHelper.produit.all()
                .then(
                    produits => {
                        const promises = produits.map(produit => {
                            return dbHelper.stock.byNom(produit.nom_produit)
                                .then(stocks => {
                                    produit.stocks = stocks;
                                    return produit;
                                });
                        });
                        return Promise.all(promises);
                    }
                )
                .then(
                    produits => {

                        //transformation récursive du format a_b en aB
                        produits = produits.map(produit => {
                            //pour chaque clés et récursivement pour chaque sous clés
                            Object.keys(produit).forEach(key => {
                                if (key.includes('_')) {
                                    const newKey = key.replace(/_./g, (match) => {
                                        return match[1].toUpperCase();
                                    });
                                    produit[newKey] = produit[key];
                                    delete produit[key];
                                }
                                if (key === 'stocks') {
                                    produit.stocks = produit.stocks.map(stock => {
                                        Object.keys(stock).forEach(key => {
                                            if (key.includes('_')) {
                                                const newKey = key.replace(/_./g, (match) => {
                                                    return match[1].toUpperCase();
                                                });
                                                stock[newKey] = stock[key];
                                                delete stock[key];
                                            }
                                        });
                                        return stock;
                                    });
                                }
                            });
                            return produit;
                        });

                        res.set('Content-type', 'application/json');
                        res.status(200).send(JSON.stringify(produits));
                        return;
                    })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );

    app.get('/inventaire/:nom_produit',
        // require('connect-ensure-login').ensureLoggedIn(),
        function (req, res) {

            dbHelper.produit.byNom(req.params.nom_produit)
                .then(
                    produit => {
                        return dbHelper.stock.byNom(produit.nom_produit)
                            .then(stocks => {
                                produit.stocks = stocks;
                                return produit;
                            });
                    }
                )
                .then(
                    produit => {

                        //transformation récursive du format a_b en aB
                        //pour chaque clés et récursivement pour chaque sous clés
                        Object.keys(produit).forEach(key => {
                            if (key.includes('_')) {
                                const newKey = key.replace(/_./g, (match) => {
                                    return match[1].toUpperCase();
                                });
                                produit[newKey] = produit[key];
                                delete produit[key];
                            }
                            if (key === 'stocks') {
                                produit.stocks = produit.stocks.map(stock => {
                                    Object.keys(stock).forEach(key => {
                                        if (key.includes('_')) {
                                            const newKey = key.replace(/_./g, (match) => {
                                                return match[1].toUpperCase();
                                            });
                                            stock[newKey] = stock[key];
                                            delete stock[key];
                                        }
                                    });
                                    return stock;
                                });
                            }
                        });


                        res.set('Content-type', 'application/json');
                        res.status(200).send(JSON.stringify(produit));
                        return;
                    })
                .catch(err => {
                    res.status(500).send('' + err);
                    return;
                });
        }
    );





    //////////////////////////////////////////////////////////////////////////////
    // CRUD UTILISATEUR TODO
    //////////////////////////////////////////////////////////////////////////////



    return app;
}
