/* eslint-env browser */
/* global Mustache, page */
'use strict';

// Le script principal de notre application single page
// Celui-ci effectue le routing coté client (et d'autres choses)

// Notre objet contexte, qui contiendra toutes les données
// pour les templates Mustache
let context = { 'logged': false };

// fonction utilitaire permettant de faire du 
// lazy loading (chargement à la demande) des templates
const templates = (() => {
    let templates = {};
    return function load(url) {
        if (templates[url]) {
            return Promise.resolve(templates[url]);
        }
        else {
            return fetch(url)
                .then(res => res.text())
                .then(text => {
                    return templates[url] = text;
                })
        }
    }
})();

// Fonction utilitaire qui permet de charger en parallèle les 
// différents "partial" (morceaux de template. Ex: header)
const loadPartials = (() => {
    let partials;

    return async function loadPartials() {
        if (!partials) {
            partials = {
                header: templates('public/views/header.mustache'),
                footer: templates('public/views/footer.mustache'),
                produit: templates('public/views/produit.mustache'),
                fournisseur: templates('public/views/fournisseur.mustache')
            };
            const promises = Object.entries(partials)
                .map(async function ([k, v]) {
                    return [k, await v];
                });
            partials = Object.fromEntries(await Promise.all(promises));
        }
        return partials;
    }
})();

// Route pour la page d'admin
page('/admin', async function () {
    // Si on n'est pas authentifé, on affiche la page de login
    if (!context.logged) {
        page('/login');
    }
    else {
        // Sinon on charge le template d'admin
        // Si on n'est pas authentifié (ex: un petit malin a changé la valeur de 'logged')
        // le serveur ne renverra tout de même pas le template puisqu'il est dans la partie 'private'
        renderTemplate(templates('private/admin.mustache'), { ...context, ariane: [{ text: 'Home', url: '/' }, { text: 'Admin' }] });
    }
});

// Route pour la page d'admin
page('/listecourse', async function () {


    updateHeaderContext(context, 'listecourse')

    renderListeCourse();
    // fonction interne de chargement et d'affichage des albums d'un artiste
    async function renderListeCourse() {


        context.produits = [
            {
                produitName: "jambon",
                produitImageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
            },
            {
                produitName: "poivre",
                produitImageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
            },
            {
                produitName: "fromage",
                produitImageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
            },
            {
                produitName: "café",
                produitImageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
            },
            {
                produitName: "thé",
                produitImageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
            },
            {
                produitName: "pain",
                produitImageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
            },
            {
                produitName: "pâtisseri",
                produitImageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
            }
        ]


        renderTemplate(templates('public/page/listecourse.mustache'), context);
    }
});



// route pour la page d'authentification des utilisateurs
page('/login', async function () {
    // pas besoin de faire de await sur cet appel puisqu'il n'y a pas d'autre 
    // traitement ensuite
    renderLoginPage(context);

    // fonction interne d'affichage de la page 
    async function renderLoginPage(context) {
        // On rend le template
        await renderTemplate(templates('public/page/login.mustache'), context);

        // Puis on ajoute l'écouteur d'évenement sur les boutons
        const cancel_btn = document.querySelector('#cancel-btn');
        cancel_btn.addEventListener('click', function () {
            page('/');
        });
        const login_btn = document.querySelector('#login-btn');
        login_btn.addEventListener('click', async function () {
            // Récupération du login et du mot de passe
            const username = document.querySelector('input[placeholder="username"]').value;
            console.log('username: ' + username);
            const password = document.querySelector('input[placeholder="password"]').value;
            let result;
            try {
                // On fait ensuite un fetch sur l'api pour s'authentifier
                result = await fetch('api/login', {
                    headers: {
                        'Accept': 'application/json',
                        'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
                    },
                    method: 'POST',
                    body: 'username=' + encodeURIComponent(username) + '&password=' + encodeURIComponent(password),
                });
            }
            catch (e) {
                console.error(e);
                return;
            }
            try {
                if (result.ok) {
                    // Si tout s'est bien passé
                    result = await result.json();
                    // Et que l'authentification s'est bien passée
                    if (result.success) {
                        // on passe à la page d'administration
                        context.logged = true;
                        page('/admin');
                    }
                    else {
                        // Sinon on réaffiche la page avec quelques infos pour expliquer ce qui n'a pas marché
                        renderLoginPage({ ...context, username, password, message: result.message });
                    }
                }
            }
            catch (e) {
                console.error(e);
                return;
            }
        });
    }
});

page('/inventaire', async function (request) {
    let response;
    updateHeaderContext(context, "inventaire");

    renderInventaire();

    // fonction interne de chargement et d'affichage des albums d'un artiste
    async function renderInventaire() {
        // On charge les données depuis l'API
        context.fournisseurs = [
            {
                nomFournisseur: "leclerc"
            },
            {
                nomFournisseur: "carrefour"
            }
        ]

        context.produits = [
            {
                nomProduit: "jambon",
                imageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
                prixUnitaire: 10,
                quantiteMin: 50,
                quantiteMax: 10,
                fournisseur: "leclerc",
                stock: [
                    {
                        idStock: 1,
                        datePeremption: "19 janvier",
                        quantite: 12,

                    },
                    {
                        idStock: 2,
                        datePeremption: "22 janvier",
                        quantite: 2,
                        critical: true
                    },
                    {
                        idStock: 3,
                        datePeremption: "23 janvier",
                        quantite: 22,

                    },
                    {
                        idStock: 4,
                        datePeremption: "22 janvier",
                        quantite: 22,

                    },
                ]
            },
            {
                nomProduit: "poivre",
                imageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
                prixUnitaire: 15,
                quantiteMin: 20,
                quantiteMax: 5,
                fournisseur: "leclerc",
            },
            {
                nomProduit: "fromage",
                imageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
                prixUnitaire: 12,
                quantiteMin: 40,
                quantiteMax: 10,
                fournisseur: "leclerc",
            },
            {
                nomProduit: "café",
                imageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
                prixUnitaire: 18,
                quantiteMin: 30,
                quantiteMax: 8,
                fournisseur: "leclerc",
            },
            {
                nomProduit: "thé",
                imageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
                prixUnitaire: 20,
                quantiteMin: 35,
                quantiteMax: 10,
                fournisseur: "leclerc",
            },
            {
                nomProduit: "pain",
                imageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
                prixUnitaire: 5,
                quantiteMin: 100,
                quantiteMax: 20,
                fournisseur: "leclerc",
            },
            {
                nomProduit: "pâtisseri",
                imageSource: "https://img.buzzfeed.com/buzzfeed-static/static/2017-03/16/20/campaign_images/buzzfeed-prod-fastlane-03/21-insanely-delicious-snacks-you-should-be-orderi-2-13042-1489710135-0_dblbig.jpg",
                prixUnitaire: 8,
                quantiteMin: 75,
                quantiteMax: 15,
                fournisseur: "leclerc",
            }
        ]


        // Rendu du template et insertion dans la page html
        await renderTemplate(templates('public/page/inventaire.mustache'), context);

        //scrool the user to the researched item
        var params = new URLSearchParams(request.querystring);
        var nomProduit = params.get('nomProduit');
        const produitAAfficher = document.getElementById('card-' + nomProduit)
        if (produitAAfficher != null) {
            produitAAfficher.scrollIntoView({ behavior: "smooth" });
        }
    }
});

// Route pour la page principale (index.html)
page('/fournisseurs', async function (request) {
    let response;
    updateHeaderContext(context, "fournisseurs");
    renderFournisseurs();
    // fonction interne de chargement et d'affichage des albums d'un artiste
    async function renderFournisseurs() {
        context.fournisseurs = [
            {
                name: "leclerc",
                address: "E.Leclerc Zone Actiparc des 3 Routes, 49120 Chemillé-en-Anjou",
                mail: "standard.chemille@scaouest.fr",
                telephone: "02 41 55 11 11",
                produitsFournisseur: [
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                ]
            },
            {
                name: "intermarché",
                address: "Intermarché. avenue Général de Gaulle. 49120 CHEMILLE",
                mail: "consommateurs.intermarche@serviceclients.mousquetaires.com",
                telephone: "0 805 101 115",
                produitsFournisseur: [
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                    {
                        descriptionProduit: "super jambon",
                        nomProduit: "jambon"
                    },
                ]
            },


        ]
        // Rendu du template et insertion dans la page html
        await renderTemplate(templates('public/page/fournisseurs.mustache'), context);

        //scrool the user to the researched item
        var params = new URLSearchParams(request.querystring);
        var nomFournisseur = params.get('fournisseur');
        console.log(nomFournisseur)
        const produitAAfficher = document.getElementById('card-' + nomFournisseur)
        if (produitAAfficher != null) {
            produitAAfficher.scrollIntoView({ behavior: "smooth" });
        }
    }
});




function updateHeaderContext(context, activePage) {
    context.pages = [
        {
            pageLink: "/",
            pageActive: activePage == "index",
            pageName: "Acceuil"
        },
        {
            pageLink: "/inventaire",
            pageActive: activePage == "inventaire",
            pageName: "Inventaire"
        },
        {
            pageLink: "/fournisseurs",
            pageActive: activePage == "fournisseurs",
            pageName: "Fournisseurs"
        },
        {
            pageLink: "/listecourse",
            pageActive: activePage == "listecourse",
            pageName: "Liste des courses"
        },
    ]
}

// Route pour la page principale (index.html)
page('/', async function () {
    updateHeaderContext(context, 'index')


    renderTemplate(templates('public/page/index.mustache'), context);

});

// On démarre le routing
page.start();

// fonction utilitaire de rendu d'un template
async function renderTemplate(template, context) {
    // On charge les partials (si pas déà chargés)
    const partials = await loadPartials();
    // On rend le template
    const rendered = Mustache.render(await template, context, partials);
    // Et on l'insère dans le body
    let body = document.querySelector('body');
    body.innerHTML = rendered;
}

